@@ -4,6 +4,7 @@ use crate::{
 };
 use crate::{Engine, ModuleType};
 use anyhow::{bail, Context, Result};
+use once_cell::sync::OnceCell;
 use std::fs;
 use std::mem;
 use std::path::Path;
@@ -11,6 +12,7 @@ use std::sync::Arc;
 use wasmparser::{Parser, ValidPayload, Validator};
 use wasmtime_environ::{ModuleEnvironment, ModuleIndex, PrimaryMap};
 use wasmtime_jit::{CompiledModule, CompiledModuleInfo, MmapVec, TypeTables};
+use wasmtime_runtime::ModuleMemFds;
 
 mod registry;
 mod serialization;
@@ -107,6 +109,11 @@ struct ModuleInner {
     types: Arc<TypeTables>,
     /// Registered shared signature for the module.
     signatures: Arc<SignatureCollection>,
+    /// A set of memfd images for memories, if any. Note that module
+    /// instantiation (hence the need for lazy init) may happen for the
+    /// same module concurrently in multiple Stores, so we use a
+    /// OnceCell.
+    memfds: OnceCell<Option<Arc<ModuleMemFds>>>,
 }
 
 impl Module {
@@ -336,7 +343,12 @@ impl Module {
         };
 
         let modules = engine.run_maybe_parallel(artifacts, |(a, b)| {
-            CompiledModule::from_artifacts(a, b, &*engine.config().profiler)
+            CompiledModule::from_artifacts(
+                a,
+                b,
+                &*engine.config().profiler,
+                engine.unique_id_allocator(),
+            )
         })?;
 
         Self::from_parts(engine, modules, main_module, Arc::new(types), &[])
@@ -531,6 +543,7 @@ impl Module {
                 artifact_upvars: modules,
                 module_upvars,
                 signatures,
+                memfds: OnceCell::new(),
             }),
         });
 
@@ -543,11 +556,13 @@ impl Module {
             module_upvars: &[serialization::SerializedModuleUpvar],
             signatures: &Arc<SignatureCollection>,
         ) -> Result<Module> {
+            let module = artifacts[module_index].clone();
             Ok(Module {
                 inner: Arc::new(ModuleInner {
                     engine: engine.clone(),
                     types: types.clone(),
-                    module: artifacts[module_index].clone(),
+                    module,
+                    memfds: OnceCell::new(),
                     artifact_upvars: artifact_upvars
                         .iter()
                         .map(|i| artifacts[*i].clone())
@@ -666,12 +681,14 @@ impl Module {
         artifact_upvars: &[usize],
         module_upvars: &[wasmtime_environ::ModuleUpvar],
         modules: &PrimaryMap<ModuleIndex, Module>,
-    ) -> Module {
-        Module {
+    ) -> Result<Module> {
+        let module = self.inner.artifact_upvars[artifact_index].clone();
+        Ok(Module {
             inner: Arc::new(ModuleInner {
                 types: self.inner.types.clone(),
                 engine: self.inner.engine.clone(),
-                module: self.inner.artifact_upvars[artifact_index].clone(),
+                module,
+                memfds: OnceCell::new(),
                 artifact_upvars: artifact_upvars
                     .iter()
                     .map(|i| self.inner.artifact_upvars[*i].clone())
@@ -687,7 +704,7 @@ impl Module {
                     .collect(),
                 signatures: self.inner.signatures.clone(),
             }),
-        }
+        })
     }
 
     pub(crate) fn compiled_module(&self) -> &Arc<CompiledModule> {
@@ -706,6 +723,16 @@ impl Module {
         &self.inner.signatures
     }
 
+    pub(crate) fn memfds(&self) -> Result<Option<&Arc<ModuleMemFds>>> {
+        Ok(self
+            .inner
+            .memfds
+            .get_or_try_init(|| {
+                ModuleMemFds::new(self.inner.module.module(), self.inner.module.wasm_data())
+            })?
+            .as_ref())
+    }
+
     /// Looks up the module upvar value at the `index` specified.
     ///
     /// Note that this panics if `index` is out of bounds since this should
@@ -942,21 +969,11 @@ struct HashedEngineCompileEnv<'a>(&'a Engine);
 #[cfg(all(feature = "cache", compiler))]
 impl std::hash::Hash for HashedEngineCompileEnv<'_> {
     fn hash<H: std::hash::Hasher>(&self, hasher: &mut H) {
-        use std::collections::BTreeMap;
-
         // Hash the compiler's state based on its target and configuration.
         let compiler = self.0.compiler();
         compiler.triple().hash(hasher);
-        compiler
-            .flags()
-            .into_iter()
-            .collect::<BTreeMap<_, _>>()
-            .hash(hasher);
-        compiler
-            .isa_flags()
-            .into_iter()
-            .collect::<BTreeMap<_, _>>()
-            .hash(hasher);
+        compiler.flags().hash(hasher);
+        compiler.isa_flags().hash(hasher);
 
         // Hash configuration state read for compilation
         let config = self.0.config();
